@@ -96,7 +96,7 @@ class LuxAI2022(ParallelEnv):
             self.py_visualizer = Visualizer(self.state)
             return True
         return False
-    def render(self, mode="human"):
+    def render(self, mode="human", **kwargs):
         """
         Renders the environment. In human mode, it can print to terminal, open
         up a graphical window, or open up some other display that a human can see and understand.
@@ -117,14 +117,11 @@ class LuxAI2022(ParallelEnv):
             self.py_visualizer.update_scene(self.state)
             VIDEO_W = 400
             VIDEO_H = 400
+            if "width" in kwargs:
+                VIDEO_W = kwargs["width"]
+            if "height" in kwargs:
+                VIDEO_H = kwargs["height"]
             return self.py_visualizer._create_image_array(self.py_visualizer.surf, (VIDEO_W, VIDEO_H))
-        elif mode == "rgb_array":
-            self._init_render()
-            self.py_visualizer.update_scene(self.state)
-            VIDEO_W = 400
-            VIDEO_H = 400
-            return self.py_visualizer._create_image_array(self.py_visualizer.surf, (VIDEO_W, VIDEO_H))
-
 
     def close(self):
         """
@@ -192,6 +189,9 @@ class LuxAI2022(ParallelEnv):
             if k not in self.agents:
                 raise ValueError(f"Invalid player {k}")
             if "faction" in a and "bid" in a:
+                if a["faction"] not in [e.name for e in FactionTypes]:
+                    failed_agents[k] = False
+                    continue
                 self.state.teams[k] = Team(
                     team_id=self.agent_name_mapping[k], agent=k, faction=FactionTypes[a["faction"]]
                 )
@@ -311,7 +311,7 @@ class LuxAI2022(ParallelEnv):
     def _handle_pickup_actions(self, actions_by_type: ActionsByType):
         for unit, pickup_action in actions_by_type["pickup"]:
             pickup_action: PickupAction
-            factory = self.state.board.get_factory_at(unit.pos)
+            factory = self.state.board.get_factory_at(self.state, unit.pos)
             pickup_amount = factory.sub_resource(pickup_action.resource, pickup_action.pickup_amount)
             # may waste resources if tried to pickup more than one can hold.
             actually_pickedup = unit.add_resource(pickup_action.resource, pickup_amount)
@@ -498,7 +498,7 @@ class LuxAI2022(ParallelEnv):
                     valid_acts, err_reason = self.action_space(agent).contains(unit_actions)
                     if not valid_acts:
                         failed_agents[agent] = True
-                        raise ValueError(f"{self.state.teams[agent]} Inappropriate action given. {err_reason}")
+                        self._log(f"{self.state.teams[agent]} Inappropriate action given. {err_reason}")
 
             # we should except that actions is always of type dict, if not then erroring here is fine
             for agent, unit_actions in actions.items():
@@ -560,7 +560,7 @@ class LuxAI2022(ParallelEnv):
             
             # Update lichen
             self.state.board.lichen -= 1
-            self.state.board.lichen = self.state.board.lichen.clip(0)
+            self.state.board.lichen = self.state.board.lichen.clip(0, self.env_cfg.MAX_LICHEN_PER_TILE)
             self.state.board.lichen_strains[self.state.board.lichen == 0] = -1
 
             # resources refining
@@ -591,16 +591,26 @@ class LuxAI2022(ParallelEnv):
         # rewards for all agents are placed in the rewards dictionary to be returned
         rewards = {}
         for agent in self.agents:
-            strain_ids = self.state.teams[agent].factory_strains
-            if failed_agents[agent]:
-                rewards[agent] = -1000
+            if agent in self.state.teams:
+                strain_ids = self.state.teams[agent].factory_strains
+                factories_left = len(self.state.factories[agent])
+                if factories_left == 0 and self.state.real_env_steps >= 0:
+                    failed_agents[agent] = True
+                    self._log(f"{agent} lost all factories")
+                if failed_agents[agent]:
+                    rewards[agent] = -1000
+                else:
+                    agent_lichen_mask = np.isin(self.state.board.lichen_strains, strain_ids)
+                    rewards[agent] = self.state.board.lichen[agent_lichen_mask].sum()
             else:
-                agent_lichen_mask = np.isin(self.state.board.lichen_strains, strain_ids)
-                rewards[agent] = self.state.board.lichen[agent_lichen_mask].sum()
+                # if this was not initialize then agent failed in step 0
+                failed_agents[agent] = True
+                rewards[agent] = -1000
 
         self.env_steps += 1
         self.state.env_steps += 1
         env_done = self.state.real_env_steps >= self.state.env_cfg.max_episode_length
+        env_done = failed_agents["player_0"] or failed_agents["player_1"] # env is done if any agent fails.
         dones = {agent: env_done or failed_agents[agent] for agent in self.agents}
 
         # generate observations
